!----------------------------------------------------------------------
!----------------------------------------------------------------------
!      A one-cell, two-substrate enzyme model 
!----------------------------------------------------------------------
!----------------------------------------------------------------------

      SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP)
!     ---------- ----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
      DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM), DFDP(NDIM,*)

      DOUBLE PRECISION s,a,s0,a0,alpha,rho,kappa
      DOUBLE PRECISION d, r, drds, drda, drdk

       s = U(1)
       a = U(2)

       s0    = PAR(1)
       a0    = PAR(2)
       alpha = PAR(3)
       rho   = PAR(4)
       kappa = PAR(5)

       d = 1+s+kappa*s**2
       r = s*a/d

       F(1) =       (s0-s) - rho*r
       F(2) = alpha*(a0-a) - rho*r

       IF(IJAC.EQ.0)RETURN

       drds = ( a*d - s*a*(1+2*kappa*s) ) / d**2
       drda = s/d
       drdk =-s**3*a/d**2

       DFDU(1,1) =-1     - rho*drds
       DFDU(1,2) =       - rho*drda
       DFDU(2,1) =       - rho*drds
       DFDU(2,2) =-alpha - rho*drda

      IF(IJAC.EQ.1)RETURN 

!      Parameter derivatives

       DFDP(1,1) = 1
       DFDP(1,2) = 0
       DFDP(1,3) = 0
       DFDP(1,4) =-R
       DFDP(1,5) =-rho*drdk

       DFDP(2,1) = 0
       DFDP(2,2) = alpha
       DFDP(2,3) = a0-a
       DFDP(2,4) =-r
       DFDP(2,5) =-rho*drdk

      END SUBROUTINE FUNC
!----------------------------------------------------------------------

      SUBROUTINE STPNT(NDIM,U,PAR,T)
!     ---------- -----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM
      DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM),PAR(*)
      DOUBLE PRECISION, INTENT(IN) :: T
      DOUBLE PRECISION s, a, s0, a0, alpha, rho, kappa

       s0    = 110
       a0    = 500
       alpha = 0.2
       rho   = 2.021628d0
       kappa = 0.1d0

       PAR(1) = s0
       PAR(2) = a0
       PAR(3) = alpha
       PAR(4) = rho
       PAR(5) = kappa

       s = 45.55974d0
       a = 177.7987d0

       U(1) = s
       U(2) = a

      END SUBROUTINE STPNT
!----------------------------------------------------------------------
      SUBROUTINE BCND
      END SUBROUTINE BCND

      SUBROUTINE ICND
      END SUBROUTINE ICND

      SUBROUTINE FOPT
      END SUBROUTINE FOPT

      SUBROUTINE PVLS
      END SUBROUTINE PVLS
!----------------------------------------------------------------------
!----------------------------------------------------------------------
